package com.a_quest.aqk2kdemo;

import android.app.Activity;
import android.app.ProgressDialog;
import android.content.res.AssetManager;
import android.media.AudioFormat;
import android.media.AudioManager;
import android.media.AudioTrack;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.content.Intent;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;

import aqkanji2koe.AqKanji2Koe;
import aquestalk.AquesTalk;	// AquesTalkクラス 別途libAquesTalk.soが必要

public class MainActivity extends Activity implements View.OnClickListener {
    AquesTalk aquestalk;
    AudioTrack audioTrack;
    private ProgressDialog dialog;	// 初期起動時の初期化処理時用


    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // 初期起動時に辞書データassets/aq_dic.zipをfileの下に展開
        copyDic();

        aquestalk = new AquesTalk();

        // ライセンスキー設定
        // ライセンス購入時のライセンス証に記載されているものを指定
        int iret = AqKanji2Koe.setDevKey("YYY-YYY-YYY");
        int iret2 = AquesTalk.setDevKey("ZZZ-ZZZ-ZZZ");
        // 使用ライセンスキーはエンドユーザが設定するので、実際はハードコーディングでなく設定画面を用意する
        int iret3 = AquesTalk.setUsrKey("UUU-UUU-UUU");

        findViewById(R.id.btnUsrDic).setOnClickListener(this);
        findViewById(R.id.btnConv).setOnClickListener(this);
        findViewById(R.id.btnPlay).setOnClickListener(this);
    }

    public void onClick(View v){
        if (v != null) {
            switch (v.getId()) {
                case R.id.btnUsrDic:
                    // UsrDicボタンが押された
                    Intent intent = new Intent(this, UsrDicActivity.class );
                    startActivity(intent);
                    break;

                case R.id.btnConv:
                    // Convertボタンが押された
                    EditText textKanji = (EditText)findViewById(R.id.editKanji);
                    String dirDic = getFilesDir().toString();	// /data/data/<package name>/files
                    String kanji = textKanji.getText().toString();
                    String strKoe = AqKanji2Koe.convert(dirDic, kanji);
                    EditText textKoe = (EditText)findViewById(R.id.editKoe);
                    textKoe.setText((CharSequence) strKoe, TextView.BufferType.SPANNABLE);
                    break;

                case R.id.btnPlay:
                    // PLAYボタンが押された
                    EditText editKoe = (EditText) findViewById(R.id.editKoe);
                    String koe = editKoe.getText().toString();
                    onPlayBtn(koe);
                    break;

                default:
                    break;
            }
        }
    }

    private void onPlayBtn(String koe) {
        // 音声合成
        aquestalk.SetPreset(AquesTalk.PRESET.F1);//声種は
        byte[] wav = aquestalk.synthe(koe);
        if(wav.length==1){//生成エラー時には,長さ１で、先頭にエラーコードが返される
            Log.v("AqK2kDemo", "AquesTalk synthe ERROR:" + wav[0]);
            Toast.makeText(this, "音声記号列が正しい？："+wav[0], Toast.LENGTH_LONG).show();
        }
        else {	// 音声出力
            if(audioTrack!=null) {// インスタンスがあれば停止/解放
                audioTrack.stop();
                audioTrack.release();
            }
            audioTrack = new AudioTrack(
                    AudioManager.STREAM_MUSIC,
                    (wav[25] << 8) + wav[24], // WAVヘッダからサンプリF1ング周波数[Hz]を取得
                    AudioFormat.CHANNEL_OUT_MONO,
                    AudioFormat.ENCODING_PCM_16BIT,
                    wav.length - 44,
                    AudioTrack.MODE_STATIC);
            audioTrack.write(wav, 44, wav.length - 44);// 44:wavのヘッダサイズ
            audioTrack.play();
        }
    }

    // 初回起動時に辞書データを展開する
    // 別スレッドで処理
    // /data/data/<app>/files/copyed.datが存在しなかったら
    // /assets/aq_dic.zip を /data/data/<app>/files/に ファイルを展開
    private void copyDic(){
        try {
            // すでに展開済み？
            String filepath = this.getFilesDir().getAbsolutePath() + "/" +  "copyed.dat";
            File file = new File(filepath);
            boolean isExists = file.exists();

            if(!isExists){//展開済みでなかったら（初期起動時）
                initDialog();
                new Thread(new Runnable(){
                    @Override
                    public void run() {
                        try{
                            AssetManager am  = getResources().getAssets();
                            InputStream is  = am.open("aq_dic.zip", AssetManager.ACCESS_STREAMING);
                            ZipInputStream zis = new ZipInputStream(is);
                            ZipEntry ze  = zis.getNextEntry();

                            int totalSize=0;
                            for(;ze != null;) {
                                String path = getFilesDir().toString() + "/" + ze.getName();
                                FileOutputStream fos = new FileOutputStream(path, false);
                                byte[] buf = new byte[8192];
                                int size = 0;
                                int posLast=0;
                                while ((size = zis.read(buf, 0, buf.length)) > -1) {
                                    fos.write(buf, 0, size);
                                    totalSize += size;
                                    int pos = totalSize*100/27220452+1;
                                    if(posLast!=pos){
                                        dialog.setProgress(pos);
                                        posLast=pos;
                                    }
                                }
                                fos.close();
                                zis.closeEntry();
                                ze  = zis.getNextEntry();
                            }
                            zis.close();
                            {// コピー完了のマークとして、copyed.datを作成
                                String filepath = getFilesDir().getAbsolutePath() + "/" +  "copyed.dat";
                                FileOutputStream fos = new FileOutputStream(filepath, false);
                                byte[] buf = new byte[1];
                                buf[0]='*';
                                fos.write(buf, 0, 1);
                                fos.close();
                            }

                        } catch(Exception e){
                            e.printStackTrace();

                        }
                        dialog.dismiss();
                    }
                }).start();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    // 初回起動時のプログレスダイアログの初期化
    protected void initDialog()
    {
        dialog = new ProgressDialog(this);	// 起動時に一度しか呼ばれないので
        dialog.setTitle("初回起動初期化中");
        dialog.setMessage("数分かかることがあります");
        dialog.setIndeterminate(false);
        dialog.setProgressStyle(ProgressDialog.STYLE_HORIZONTAL);
        dialog.setMax(100);
        dialog.setCancelable(false);
        dialog.show();
    }

    // EditTextからフォーカスが抜けたときにソフトキーボードを隠す設定
    protected void setEditText(int id)
    {
        EditText et = (EditText)findViewById(id);
        et.setOnFocusChangeListener(new View.OnFocusChangeListener(){

            @Override
            public void onFocusChange(View v, boolean flag){
                if(flag == false){
//                    InputMethodManager imm = (InputMethodManager)getSystemService(Context.INPUT_METHOD_SERVICE);
                    InputMethodManager imm = (InputMethodManager)getSystemService(MyApplication.getInstance().INPUT_METHOD_SERVICE);
                    imm.hideSoftInputFromWindow(v.getWindowToken(),0);
                }
            }
        });
    }

}
